# Script file called by the Azure custom script extension during Windows VM deployment.
# Preps the machine for running the VM agent on reboot.
# Errors here should fail the script and exception will be caught in outer script and fail the custom script extension deployment.
param
(
    [string] $vsoAgentDir,
    # Base 64 encoded (UTF8) json blob. No nested json as this is parsed into a hashtable<string, string>
    [string] $base64ParametersBlob,
    [string] $logFile
)

$ErrorActionPreference = "Stop"

function Log ([string] $message)
{
    Write-Host $message
    $message = "[$(Get-Date -Format o)] $message $([Environment]::NewLine)"
    [System.IO.File]::AppendAllText($logFile, $message)
}

function ThrowIfKeyNotSet($hashtable, $key)
{
    Log "Ensuring parameter '$key' exists..."
    if (-Not $parameters[$key])
    {
        throw "Required parameter '$key' not found.'"
    }
}

Log "Parsing parameters blob..."
$parametersJsonBlob = [System.Text.Encoding]::UTF8.GetString([System.Convert]::FromBase64String($base64ParametersBlob))

$parameters = @{}
(ConvertFrom-Json $parametersJsonBlob).psobject.properties | Foreach { $parameters[$_.Name] = $_.Value }

# Ensure we have parameters we need.
ThrowIfKeyNotSet $parameters "inputQueueName"
ThrowIfKeyNotSet $parameters "inputQueueUrl"
ThrowIfKeyNotSet $parameters "inputQueueSasToken"
ThrowIfKeyNotSet $parameters "vmToken"
ThrowIfKeyNotSet $parameters "resourceId"
ThrowIfKeyNotSet $parameters "serviceHostName"
ThrowIfKeyNotSet $parameters "userName"
ThrowIfKeyNotSet $parameters "visualStudioInstallationDir"

$configIni = [IO.Path]::Combine($vsoAgentDir, "config.ini")

if (Test-Path $configIni -PathType Leaf)
{
    Remove-Item $configIni -Force
}

Log "Writing config.ini..."
Add-Content $configIni "[ENVAGENTSETTINGS]"
Add-Content $configIni "INPUTQUEUENAME=$($parameters['inputQueueName'])"
Add-Content $configIni "INPUTQUEUEURL=$($parameters['inputQueueUrl'])"
Add-Content $configIni "INPUTQUEUESASTOKEN=$($parameters['inputQueueSasToken'])"
Add-Content $configIni "[HEARTBEATSETTINGS]"
Add-Content $configIni "VMTOKEN=$($parameters['vmToken'])"
Add-Content $configIni "RESOURCEID=$($parameters['resourceId'])"
Add-Content $configIni "SERVICEHOSTNAME=$($parameters['serviceHostName'])"

# TODO: janraj, following lines are temporary hacks to light up nexus.

# TODO: vso vm agent should be running as a service. vso cli should be launched by the vso vm agent.
# Launching an interactive app on a interactive login from a service has proved difficult so far.
# Currently when this script is run as Local System Account, it will author a cmd file in the startup folder 
# and do a restart of the box. Once the windows vm restarts, vsonline user will be auto login-ed and also 
# the startup cmd will be executed. 
Log "Write start up file..."
$visualStudioInstallationDir = $parameters["visualStudioInstallationDir"]
$startUpFile = "C:\Users\$($parameters['userName'])\AppData\Roaming\Microsoft\Windows\Start Menu\Programs\Startup\vsoagent.cmd"

if (Test-Path $startUpFile -PathType Leaf)
{
    Remove-Item $startUpFile -Force
}

Log "Configuring git credential plugin"
Add-Content $startUpFile 'git config --system --unset credential.helper'
Add-Content $startUpFile 'git config --global credential.helper "!f() { c:/vsonline/vsoagent/bin/vso gitCredential $*; }; f"'

# The following is to add the VSLS-Agent.exe to the firewall exceptions.
Add-Content $startUpFile "netsh advfirewall firewall add rule name=`"VsoVSLS`" dir=in action=allow program=`"$vsoAgentDir\vsls-agent.exe`" enable=yes"

Add-Content $startUpFile "cd $vsoAgentDir"
Add-Content $startUpFile "$vsoAgentDir\vso.exe vmagent"

# TODO: Installs Hyper-V module to get VHD mounting capability in powershell. This should move to VHD creation as well.
Log "Enable the Hyper-V powershell module..."
& dism /online /enable-feature /featurename:Microsoft-Hyper-V /Quiet /NoRestart /all
& bcdedit /set hypervisorlaunchtype off

# Installs CredentialManager for storing credentials to access the storage file share.
Install-Module CredentialManager -force

# TODO: Once the service can launch vso cli as an interactive process. This will be no longer needed.
# Schedule a restart to run the start up program which starts the VM agent.
Log "Scheduling restart..."
& $env:SystemRoot\system32\shutdown.exe -r -f -t 60

Log "Finished WindowsInit.ps1."
